@@ -43,7 +43,8 @@ AnyEvent::IRC::Connection - An IRC connection abstraction
 =head1 DESCRIPTION
 
 The connection class. Here the actual interesting stuff can be done,
-such as sending and receiving IRC messages.
+such as sending and receiving IRC messages. And it also handles
+TCP connecting and even enabling of TLS.
 
 Please note that CTCP support is available through the functions
 C<encode_ctcp> and C<decode_ctcp> provided by L<AnyEvent::IRC::Util>.
@@ -52,9 +53,9 @@ C<encode_ctcp> and C<decode_ctcp> provided by L<AnyEvent::IRC::Util>.
 
 =over 4
 
-=item B<new>
+=item $con = AnyEvent::IRC::Connection->new ()
 
-This constructor does take no arguments.
+This constructor doesn't take any arguments.
 
 =cut
 
@@ -62,18 +63,28 @@ sub new {
   my $this = shift;
   my $class = ref($this) || $this;
 
-  my $self = { heap => {} };
+  my $self = $class->SUPER::new (@_);
 
   bless $self, $class;
 
+  $self->reg_cb (
+     ext_after_send => sub {
+        my ($self, $mkmsg_args) = @_;
+        $self->send_raw (mk_msg (@$mkmsg_args));
+     }
+  );
+
   return $self;
 }
 
-=item B<connect ($host, $port)>
+=item $con->connect ($host, $port)
 
 Tries to open a socket to the host C<$host> and the port C<$port>.
 If an error occurred it will die (use eval to catch the exception).
 
+If you want to connect via TLS/SSL you have to call the C<enable_ssl>
+method before to enable it.
+
 =cut
 
 sub connect {
@@ -98,6 +109,7 @@ sub connect {
       $self->{socket} =
          AnyEvent::Handle->new (
             fh => $fh,
+            ($self->{enable_ssl} ? (tls => 'connect') : ()),
             on_eof => sub {
                $self->disconnect ("EOF from server $host:$port");
             },
@@ -119,7 +131,18 @@ sub connect {
    };
 }
 
-=item B<disconnect ($reason)>
+=item $con->enable_ssl ()
+
+This method will enable SSL for new connections that are initiated by C<connect>.
+
+=cut
+
+sub enable_ssl {
+   my ($self) = @_;
+   $self->{enable_ssl} = 1;
+}
+
+=item $con->disconnect ($reason)
 
 Unregisters the connection in the main AnyEvent::IRC object, closes
 the sockets and send a 'disconnect' event with C<$reason> as argument.
@@ -133,7 +156,7 @@ sub disconnect {
    $self->event (disconnect => $reason);
 }
 
-=item B<is_connected>
+=item $con->is_connected
 
 Returns true when this connection is connected.
 Otherwise false.
@@ -145,7 +168,7 @@ sub is_connected {
    $self->{socket} && $self->{connected}
 }
 
-=item B<heap ()>
+=item $con->heap ()
 
 Returns a hash reference that is local to this connection object
 that lets you store any information you want.
@@ -157,7 +180,7 @@ sub heap {
    return $self->{heap};
 }
 
-=item B<send_raw ($ircline)>
+=item $con->send_raw ($ircline)
 
 This method sends C<$ircline> straight to the server without any
 further processing done.
@@ -171,7 +194,7 @@ sub send_raw {
    $self->{socket}->push_write ($ircline . "\015\012");
 }
 
-=item B<send_msg ($command, @params)>
+=item $con->send_msg ($command, @params)
 
 This function sends a message to the server. C<@ircmsg> is the argument list
 for C<AnyEvent::IRC::Util::mk_msg (undef, $command, @params)>.
@@ -181,8 +204,8 @@ for C<AnyEvent::IRC::Util::mk_msg (undef, $command, @params)>.
 sub send_msg {
    my ($self, @msg) = @_;
 
+   $self->event (send => [undef, @msg]);
    $self->event (sent => undef, @msg);
-   $self->send_raw (mk_msg (undef, @msg));
 }
 
 sub _feed_irc_data {
@@ -205,13 +228,13 @@ registering event callbacks.
 
 =over 4
 
-=item B<connect>
+=item connect => $error
 
 This event is generated when the socket was successfully connected
 or an error occurred while connecting. The error is given as second
-argument to the callback then.
+argument (C<$error>) to the callback then.
 
-=item B<disconnect $reason>
+=item disconnect => $reason
 
 This event will be generated if the connection is somehow terminated.
 It will also be emitted when C<disconnect> is called.
@@ -220,19 +243,44 @@ a clue about why the connection terminated.
 
 If you want to reestablish a connection, call C<connect> again.
 
-=item B<sent @ircmsg>
+=item send => $ircmsg
+
+Emitted when a message is about to be sent. C<$ircmsg> is an array reference
+to the arguments of C<mk_msg> (see L<AnyEvent::IRC::Util>). You
+may modify the array reference to change the message or even intercept it
+completely by calling C<stop_event> (see L<Object::Event> API):
+
+   $con->reg_cb (
+      send => sub {
+         my ($con, $ircmsg) = @_;
+
+         if ($ircmsg->[1] eq 'NOTICE') {
+            $con->stop_event; # prevent any notices from being sent.
+
+         } elsif ($ircmsg->[1] eq 'PRIVMSG') {
+            $ircmsg->[-1] =~ s/sex/XXX/i; # censor any outgoing private messages.
+         }
+      }
+   );
+
+=item sent => @ircmsg
 
 Emitted when a message (C<@ircmsg>) was sent to the server.
 C<@ircmsg> are the arguments to C<AnyEvent::IRC::Util::mk_msg>.
 
-=item B<'*' $msg>
+=item irc_* => $msg
 
-=item B<read $msg>
+=item irc_<lowercase command> => $msg
+
+=item read => $msg
 
 Emitted when a message (C<$msg>) was read from the server.
 C<$msg> is the hash reference returned by C<AnyEvent::IRC::Util::parse_irc_msg>;
 
-=item B<buffer_empty>
+Note: '<lowercase command>' stands for the command of the message in
+(ASCII) lower case.
+
+=item buffer_empty
 
 This event is emitted when the write buffer of the underlying connection
 is empty and all data has been given to the kernel. See also C<samples/notify>
@@ -254,7 +302,7 @@ L<AnyEvent::IRC::Client>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Robin Redeker, all rights reserved.
+Copyright 2006-2009 Robin Redeker, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
